﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;

namespace PeteBrown.SilverlightChat
{
    public partial class Page : UserControl
    {
        private SocketClient _socketClient;

        public Page()
        {
            InitializeComponent();

            _socketClient = new SocketClient();

            SendMessage.Click += new RoutedEventHandler(SendMessage_Click);

            Loaded += new RoutedEventHandler(Page_Loaded);
        }

        void OnChatMessageReceived(object sender, ChatMessageEventArgs e)
        {
            ShowMessage(e.Message);
        }

        void Page_Loaded(object sender, RoutedEventArgs e)
        {
            _socketClient.Connect();
            _socketClient.ChatMessageReceived += 
                new EventHandler<ChatMessageEventArgs>(OnChatMessageReceived);
        }

        void SendMessage_Click(object sender, RoutedEventArgs e)
        {
            _socketClient.SendMessage(MessageText.Text.Trim());
        }

        private void ShowMessage(string message)
        {
            if (message.Trim() != "")
            {
                ChatBalloon balloon = new ChatBalloon(message);

                balloon.Visibility = Visibility.Collapsed;
                balloon.RemoveMe += new EventHandler(balloon_RemoveMe);

                Random rnd = new Random();

                balloon.SetValue(Canvas.LeftProperty, 
                    rnd.Next((int)(ChatMessageContainer.ActualWidth - balloon.ActualWidth)));
                balloon.SetValue(Canvas.TopProperty, 
                    rnd.Next((int)(ChatMessageContainer.ActualHeight - balloon.ActualHeight * 2)));

                ChatMessageContainer.Children.Add(balloon);
                balloon.Visibility = Visibility.Visible;

            }
        }

        void balloon_RemoveMe(object sender, EventArgs e)
        {
            ChatMessageContainer.Children.Remove((UIElement)sender);
        }

    }
}
